#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>
#include <sys/mman.h>

#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
//----------------------------------------------------------------------------

int			m_fd = 0;
caddr_t		m_dmap;
//----------------------------------------------------------------------------

void setOverlayType( int iOverlayType )
{
	ioctl( m_fd, _IOW( 'd', 12, int ), &iOverlayType );
}
//----------------------------------------------------------------------------

void initDisplay()
{
	m_fd = open( "/dev/display", O_RDWR | O_SYNC );
	if( m_fd == -1 )
	{
		printf( "Error: Could not open display\n" );
	}

	m_dmap = (caddr_t)mmap( 0, 2048, PROT_READ | PROT_WRITE, MAP_SHARED, m_fd, 0 );
	if( m_dmap == (caddr_t)-1 )
	{
		printf( "Error: Could not mmap display\n" );
	}

	// turn display on
	ioctl( m_fd, _IOW( 'd', 1, int ), 1 );

	// enable overlay mode
	setOverlayType( 1 );
}
//----------------------------------------------------------------------------

void deinitDisplay()
{
	// disable overlay mode
	setOverlayType( 0 );

	// turn display off
//	ioctl( m_fd, _IOW( 'd', 1, int ), 0 );

	if( munmap( m_dmap, 2048 ) == -1 || close( m_fd ) == -1 )
	{
		printf( "Error: Could not unmap display\n" );
	}
}
//----------------------------------------------------------------------------

unsigned char buffer[ 128 * 32 ] = { 0 };

void blit()
{
	unsigned char	c = 0;
	int				i = 0;
	int				j = 0;
	int				iBytes = 128 * 32;
	unsigned char*	pBytes = &buffer[0];

	for( i = 0; i < iBytes; ++i )
	{
		c = c & 0x0F;
		c |= *pBytes++ << 4;
		if( i & 1 )
			m_dmap[ j++ ] = c;
		c >>= 4;
	}

	// put screen into display queue
	ioctl( m_fd, _IO( 'd', 6 ) );
}
//----------------------------------------------------------------------------

int main( int iArgCount, void** ppArgList )
{
	int i = 0;
	int iRunCount = 15;
	
	printf( "This program enables overlay display mode from kernel and blits\n" );
	printf( "random image to screen once a second for 15 seconds and then exits\n" );
	
	initDisplay();
	while( iRunCount )
	{
		sleep( 1 );
		
		for( i = 0; i < 128*32; ++i )
			buffer[ i ] = rand()%4;
		
		blit();
		iRunCount--;
	}
	
	printf( "\nExiting from overlay_display...\n" );
	deinitDisplay();
	return 1;
}
//----------------------------------------------------------------------------
